/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2006, Nickolay V. Shmyrev
 *
 * Nickolay V. Shmyrev <nshmyrev@yandex.ru>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief This application allows asterisk to establish talk between
 * two endpoints.
 *
 * \author Nickolay V. Shmyrev <nshmyrev@yandex.ru>
 * 
 * This is a skeleton for development of an Asterisk application 
 * \ingroup applications
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 10691 $")

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/features.h"
#include "asterisk/app.h"

static char *tdesc = "Connect two endpoints Application";
static char *app = "Connect";
static char *synopsis = 
"Application that allows to establish talk between two endpoints.";

static char *descrip = 
"Usage: Connect(Technology1/resource1&Technology2/resource2)\n"
"Connects two endpoints together. Dials first resource and waits for\n"
"answer. Then it dials the second resource and bridge them togather.\n";

LOCAL_USER_DECL;

static int app_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	struct localuser *u;
	char *parse;
	int cause;

	struct ast_channel *first_chan;
	struct ast_channel *second_chan;

	char *first_tech, *second_tech;
	char *first_number, *second_number;

	struct ast_bridge_config config;

	LOCAL_USER_ADD(u);

	if (!(parse = ast_strdupa(data))) {
		LOCAL_USER_REMOVE(u);
		return -1;
	}
		
	first_tech = strsep(&parse, "/");
	first_number = strsep (&parse, "&");
	second_tech = strsep (&parse, "/");
	second_number = parse;

	first_chan = ast_request_and_dial(first_tech, chan->nativeformats, first_number, 30000, &cause, chan->cid.cid_num, chan->cid.cid_name);

	if(first_chan) {
		if(first_chan->_state == AST_STATE_UP) {
			ast_verbose(VERBOSE_PREFIX_4 "Channel %s was answered.\n", first_chan->name);
		} else {
			ast_verbose(VERBOSE_PREFIX_4 "Channel %s was never answered.\n", first_chan->name);
			ast_hangup(first_chan);
			LOCAL_USER_REMOVE(u);
			return -1;
		}
	} else {
		ast_log(LOG_WARNING, "Unable to allocate channel %s.\n", first_tech);
		LOCAL_USER_REMOVE(u);
		return -1; 
	}
	
	second_chan = ast_request_and_dial(second_tech, first_chan->nativeformats, second_number, 30000, &cause, first_chan->cid.cid_num, first_chan->cid.cid_name);

	if(second_chan) {
		if(second_chan->_state == AST_STATE_UP) {
			ast_verbose(VERBOSE_PREFIX_4 "Channel %s was answered.\n", second_chan->name);
		} else {
			ast_verbose(VERBOSE_PREFIX_4 "Channel %s was never answered.\n", second_chan->name);
			ast_hangup(second_chan);
			ast_hangup(first_chan);
			LOCAL_USER_REMOVE(u);
			return -1;
		}
	} else {
		ast_log(LOG_WARNING, "Unable to allocate channel %s.\n", second_tech);
		ast_hangup(first_chan);
		LOCAL_USER_REMOVE(u);
		return -1; 
	}

	/* Make sure channels are compatible */
	res = ast_channel_make_compatible(first_chan, second_chan);
	if (res < 0) {
		ast_log(LOG_WARNING, "Had to drop call because I couldn't make %s compatible with %s\n", first_chan->name, second_chan->name);

		ast_hangup(first_chan);
		ast_hangup(second_chan);

		LOCAL_USER_REMOVE(u);
		return -1;
	}

	ast_answer (chan);
	
	memset(&config,0,sizeof(struct ast_bridge_config));
	res = ast_bridge_call(first_chan, second_chan, &config);
	
	ast_hangup (first_chan);
	ast_hangup (second_chan);
	
	LOCAL_USER_REMOVE(u);

	return res;
}

int unload_module(void)
{
	int res;

	res = ast_unregister_application(app);

	STANDARD_HANGUP_LOCALUSERS;

	return res;	
}

int load_module(void)
{
	return ast_register_application(app, app_exec, synopsis, descrip);
}


char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;

	STANDARD_USECOUNT(res);

	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
